@@ -5,7 +5,7 @@ module Unicode
   module DisplayWidth
     DEPTHS = [0x10000, 0x1000, 0x100, 0x10].freeze
 
-    def self.of(string, ambiguous = 1, overwrite = {})
+    def self.of(string, ambiguous = 1, overwrite = {}, options = {})
       res = string.codepoints.inject(0){ |total_width, codepoint|
         index_or_value = INDEX
         codepoint_depth_offset = codepoint
@@ -19,8 +19,27 @@ module Unicode
         total_width + (overwrite[codepoint] || width || 1)
       }
 
+      res -= emoji_extra_width_of(string, ambiguous, overwrite) if options[:emoji]
       res < 0 ? 0 : res
     end
+
+    def self.emoji_extra_width_of(string, ambiguous = 1, overwrite = {}, _ = {})
+      require "unicode/emoji"
+
+      extra_width = 0
+      modifier_regex = /[#{ Unicode::Emoji::EMOJI_MODIFIERS.pack("U*") }]/
+      zwj_regex = /(?<=#{ [Unicode::Emoji::ZWJ].pack("U") })./
+
+      string.scan(Unicode::Emoji::REGEX){ |emoji|
+        extra_width += 2 * emoji.scan(modifier_regex).size
+
+        emoji.scan(zwj_regex){ |zwj_succ|
+          extra_width += self.of(zwj_succ, ambiguous, overwrite)
+        }
+      }
+
+      extra_width
+    end
   end
 end
 
